/*
    SPDX-FileCopyrightText: 2024-2034 Sergey V Turchin <zerg@altlinux.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include <sysfs/libsysfs.h>

static int __have_nvidia_prime_state = -1;

bool __nvidia_prime_available(bool o_verbose=false) {
	if( __have_nvidia_prime_state >= 0 ) {
	    return __have_nvidia_prime_state > 0;
	}

	// find second videocard
	int primary_videocards = 0;
	int nvidia_primecards = 0;
	struct sysfs_bus *bus;
	bus = sysfs_open_bus("pci");
	if( bus != NULL )
	{
	    struct dlist *devlist;
	    struct sysfs_device *curdev;
	    devlist = sysfs_get_bus_devices(bus);
	    if( devlist )
	    {
		dlist_for_each_data(devlist, curdev, struct sysfs_device)
		{
		    struct sysfs_attribute *class_attr;
		    class_attr = sysfs_get_device_attr(curdev, "class");
		    if( class_attr != NULL )
		    {
			long int class_id;
			class_id = strtol(class_attr->value,NULL,16);
			if( class_id == 0x030000 || class_id == 0x030200 )
			{
			    struct sysfs_attribute *vendor_attr;
			    struct sysfs_attribute *device_attr;
			    vendor_attr = sysfs_get_device_attr(curdev, "vendor");
			    device_attr = sysfs_get_device_attr(curdev, "device");
			    if( vendor_attr != NULL && device_attr != NULL )
			    {
				unsigned int vendor_id, device_id;
				vendor_id = strtol(vendor_attr->value,NULL,16);
				device_id = strtol(device_attr->value,NULL,16);
				if( vendor_id == 0x10de )
				{
				    if( device_id < 0x0FC6 ) {
					// driver < 435 don't support PRIME
					if( o_verbose )
					    qWarning("Found old NVIDIA controller %08x:%08x", vendor_id, device_id);
					__have_nvidia_prime_state = 0;
					return __have_nvidia_prime_state > 0;
				    } else if( class_id == 0x030000 ) {
					if( primary_videocards <= 0 ) {
					    primary_videocards++;
					    if( o_verbose )
						qWarning("Found NVIDIA VGA controller %08x:%08x", vendor_id, device_id);
					    continue;
					} else {
					    nvidia_primecards++;
					    if( o_verbose )
						qWarning("Found NVIDIA VGA controller (PRIME) %08x:%08x", vendor_id, device_id);
					    continue;
					}
				    } else if( class_id == 0x030200 ) {
					nvidia_primecards++;
					if( o_verbose )
					    qWarning("Found NVIDIA 3D controller %08x:%08x", vendor_id, device_id);
					continue;
				    }
				}
				else
				{
				    if( class_id == 0x030000 )
				    {
					primary_videocards++;
					switch(vendor_id)
					{
					    case 0x8086:
						if( o_verbose )
						    qWarning("Found Intel VGA controller %08x:%08x", vendor_id, device_id);
						break;
					    case 0x1002:
						if( o_verbose )
						    qWarning("Found AMD VGA controller %08x:%08x", vendor_id, device_id);
						break;
					    default:
						if( o_verbose )
						    qWarning("Found other VGA controller %08x:%08x", vendor_id, device_id);
						break;
					}
				    }
				}
			    } else  {
				if( o_verbose )
				    qWarning("Found unrecognized 0x03 class device");
			    }
			}
		    }
		}
	    }
	}
	if( o_verbose )
	    qWarning("Primary<%d> PRIME<%d>", primary_videocards, nvidia_primecards);
	__have_nvidia_prime_state = nvidia_primecards;
	return __have_nvidia_prime_state > 0;
}
